#define WIFI_RESET    8
#define WIFI_ENABLE   9
#define SW3           12
#define LED3          13
#define BUFF_SIZE     256
#define HOST_NAME_LEN 14
#define REQUEST_PATH_LEN 10

char _rxBuffer[BUFF_SIZE];
int _indexBuf = 0;
int push = 0;

const char WIFI_SSID[] = "SSID";
const char WIFI_PASS[] = "PASSWORD";

const char HOST_NAME[HOST_NAME_LEN + 1] = "dl.bizright.jp";
const char REQUEST_PATH[REQUEST_PATH_LEN + 1] = "/bd/sample";

const char CONTENT_SEPARATOR[] = "\r\n\r\n";

void setup() {

  pinMode(WIFI_RESET, OUTPUT);
  pinMode(WIFI_ENABLE, OUTPUT);
  pinMode(SW3, INPUT_PULLUP);
  pinMode(LED3, OUTPUT);

  Serial.begin(115200);

  _indexBuf = 0;
  _rxBuffer[0] = '\0';

  delay(3000);

  digitalWrite(WIFI_ENABLE, HIGH);

  Serial.print(F("ATE0"));
  Serial.print("\r\n");
  waitResponse(500);

  Serial.print(F("AT+CWMODE_CUR=1"));
  Serial.print("\r\n");
  waitResponse(500);

  Serial.print(F("AT+CIPMUX=1"));
  Serial.print("\r\n");
  waitResponse(500);

  digitalWrite(LED3, HIGH);
}

void loop() {
  if (digitalRead(SW3) == LOW) {
    digitalWrite(LED3, LOW);
    Serial.print(F("AT+CWJAP_CUR=\""));
    Serial.print(WIFI_SSID);
    Serial.print(F("\",\""));
    Serial.print(WIFI_PASS);
    Serial.print(F("\""));
    Serial.print("\r\n");
    waitResponse(10000);

    Serial.print(F("AT+CIPSTART=0,\"TCP\",\""));
    Serial.print(HOST_NAME);
    Serial.print(F("\",80"));
    Serial.print("\r\n");
    waitResponse(10000);

    int dataSize = 46 + REQUEST_PATH_LEN + 1 + HOST_NAME_LEN;
    Serial.print(F("AT+CIPSEND=0,"));
    Serial.print(dataSize);
    Serial.print("\r\n");
    if (waitResponse(500)) {
      push++;
      if (push > 2) {
        push = 1;
      }
      char push_char[1];
      sprintf(push_char, "%d", push);
      Serial.print(F("GET "));
      Serial.print(REQUEST_PATH);
      Serial.print(push_char);
      Serial.print(F(" HTTP/1.1\r\nHost: "));
      Serial.print(HOST_NAME);
      Serial.print(F("\r\nUser-Agent: arduino\r\n\r\n"));
      waitResponse(1000);

      waitReceiveWiFiData(1000);
      char *res = strrstr(_rxBuffer, CONTENT_SEPARATOR);

      if (atoi(res) == 1) {
        digitalWrite(LED3, HIGH);
        delay(1000);
        digitalWrite(LED3, LOW);
      } else if (atoi(res) == 2) {
        digitalWrite(LED3, HIGH);
        delay(1000);
        digitalWrite(LED3, LOW);
        delay(1000);
        digitalWrite(LED3, HIGH);
        delay(1000);
        digitalWrite(LED3, LOW);
      }
      waitResponse(1000);
    } else {
      Serial.print(F("AT+CIPCLOSE="));
      Serial.print(0);
      Serial.print("\r\n");
      waitResponse(1000);
    }

    Serial.print(F("AT+CWQAP"));
    Serial.print("\r\n");
    waitResponse(1000);
    digitalWrite(LED3, HIGH);
  }
}

boolean receive() {
  char c;
  if (Serial.available()) {
    c = Serial.read();
    if (c >= 0 && c <= 0x80 && c != '\0' && c != '\t' && c != '\v') {
      if ((_indexBuf + 1) >= BUFF_SIZE) {
        _indexBuf = 0;
        _rxBuffer[0] = '\0';
      }
      _rxBuffer[_indexBuf] = c;
      _rxBuffer[_indexBuf + 1] = '\0';
      _indexBuf++;
      return true;
    }
  }
  return false;
}

boolean waitResponse(unsigned long waitMillis) {
  boolean result = false;
  char c;
  unsigned long currentMillis, previousMillis;
  currentMillis = millis();
  previousMillis = currentMillis;

  while (currentMillis - previousMillis < waitMillis) {
    if (receive()) {
      if ((strstr(_rxBuffer, "OK") != NULL) || (strstr(_rxBuffer, "SEND OK") != NULL)) {
        result = true;
        break;
      }
      else if ((strstr(_rxBuffer, "ERROR") != NULL) || (strstr(_rxBuffer, "FAIL") != NULL)) {
        break;
      }
    }
    currentMillis = millis();
  }
  _indexBuf = 0;
  _rxBuffer[0] = '\0';
  delay(10);
  return result;
}

int waitReceiveWiFiData(unsigned long waitMillis) {
  int result = 0;
  char c;
  unsigned long currentMillis, previousMillis;
  currentMillis = millis();
  previousMillis = currentMillis;
  while (currentMillis - previousMillis < waitMillis) {
    receive();
    currentMillis = millis();
  }
  return _indexBuf;
}

char * strrstr (const char *string, const char *pattern) {
  const char *last = '\0';
  for ( const char *p = string; '\0' != (p = strstr( p, pattern )); ++p ) {
    last = p;
    if ( '\0' == *p ) {
      return (char *)last;
    }
  }
  return (char *)last;
}

